'' =================================================================================================
''
''   File....... jm_field_parse_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 02 NOV 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28


con

  #-1, ON, OFF                                                   ' digital control

  #0, LSBFIRST, MSBFIRST    


obj

' main                                                           ' * master Spin cog
  time : "jm_time"                                               '   timing and delays
  io   : "jm_io_basic"                                           '   essential io
  term : "jm_fullduplexserial"                                   ' * serial IO for terminal

' * uses cog when loaded


var

  long  a                                                        ' these are crappy var names
  long  b                       
  long  c


dat

  TestInput     byte    "1A3000B200", 0


pub main | pos

  setup

  time.pause(2000)                                               ' time to open pst

  c := str2dec(@TestInput[0], 10)

  pos := find("A", @TestInput) + 1                               ' field follows (+1) letter
  a := str2dec(@TestInput[pos], 10)                              ' allow max positive size (posx)

  pos := find("B", @TestInput) + 1
  b := str2dec(@TestInput[pos], 10)

  term.tx(term#CLS)

  term.str(string("C = "))
  term.dec(c)
  term.tx(term#CR)  

  term.str(string("A = "))
  term.dec(a)
  term.tx(term#CR)

  term.str(string("B = "))
  term.dec(b)
  term.tx(term#CR)   
  
  repeat
    waitcnt(0)


pub setup

'' Setup IO and objects for application

  time.start                                                     ' setup timing & delays

  io.start(0, 0)                                                 ' clear all pins (master cog)

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal


con

  { --------------- }
  {  S U P P O R T  }
  { --------------- }


pub find(ch, p_str) | idx

'' Find position of char ch in string at p_str
'' -- returns -1 if not found

  repeat idx from 0 to strsize(p_str)-1
    if (byte[p_str][idx] == ch)
      return idx

  return -1


pub str2dec(p_str, len) | ch, value 

'' Convert string at p_str to decimal value
'' -- max length is len characters

  value := 0                                                     ' initialize result
  
  repeat len
    ch := byte[p_str++]                                          ' get a character
    if ((ch => "0") and (ch =< "9"))                             ' valid digit?
      value := (value * 10) + (ch - "0")                         ' ASCII --> decimal
    else
      quit
  
  return value


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}